import javafx.application.Application;
import javafx.stage.Stage;
import javafx.util.Duration;
import javafx.scene.Scene;
import javafx.scene.Group;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;
import javafx.scene.paint.Color;
import javafx.scene.transform.Affine;
import javafx.animation.Animation;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;

// Animation of spinning bouncing head

public class SpinDemo extends Application 
{
	final String appName = "SpinDemo";
	final int FPS = 30; // frames per second
	final int WIDTH = 500;
    final int HEIGHT = 400;
    
    double x,y,th,dx,dy,dth; //position, rotation, and deltas
    // th (short for theta) is the rotation angle in radians
    
    Image backdrop;
    Image head;
    
    void initialize(){
    	//init ball vars
        x = 20; y = 50; dx = 3.4; dy = -6.2;
        th = 0.0; dth = 6.0;
        
        backdrop = new Image( "bd.gif" );
        head   = new Image( "head.gif" );
    }
    
    void update(){
    	  x += dx; y += dy;
    	  th += dth;
    	  if (th < 0){
    		  th += 360;
    	  } else if (th > 360){
    		  th -= 360;
    	  }
    	  // Check for bounces
    	  if ((x-20) < 0){
    		  x = -x + 40;
    		  dx = -dx;
    		  dth = -dth;
    		  wiggleDeltas();
    	  } else if ((x+20) > WIDTH){
    		  x  = 2*WIDTH - x - 40;
    		  dx = -dx;
    		  dth = -dth;
    		  wiggleDeltas();
    	  }
    	  if ((y-20) < 0){
    		  y = -y + 40;
    		  dy = -dy;
    		  dth = -dth;
    		  wiggleDeltas();
    	  } else if ((y+20) > HEIGHT){
    		  y = 2*HEIGHT - y - 40;
    		  dy = -dy;
    		  dth = -dth;
    		  wiggleDeltas();
    	  }
      }
      
    void wiggleDeltas(){
    	  // Add a little random "noise" to
    	  // the delta variables
    	  double scale;
    	  // Get a multiplier between 0.95 and 1.05
    	  scale = 0.95 + 0.1*Math.random();
    	  dx *= scale;
    	  scale = 0.95 + 0.1*Math.random();
    	  dy *= scale;
    	  scale = 0.95 + 0.1*Math.random();
    	  dth *= scale;
      }
    
    public void render(GraphicsContext gc){
    	//System.out.println("render called:"+th);
    	// Draw non-translated stuff
        gc.setFill(Color.CYAN);
        gc.fillRect(0,0,WIDTH,HEIGHT);
    	gc.setStroke(Color.BLACK);
    	gc.strokeRect(0,0,WIDTH-1,HEIGHT-1);

        gc.drawImage(backdrop, 100, 60);

        //Use an Affine transform to position the ball
        gc.save();
        Affine move = new Affine();
    	//These next three lines modify the image
    	// in reverse order (because of the way transforms
    	// compose).  So, we take our image and translate
    	// it 20 units up and left so that the center of
    	// the image is at (0,0).  Then we rotate (so the
    	// image rotates about its center).
    	// Then we move the rotated image to where we really
    	// want it.
        move.appendTranslation(x,y);
        move.appendRotation(th);
        move.appendTranslation(-20,-20);
        //draw the image using the specified transform
        gc.setTransform(move);
        gc.drawImage(head, 0, 0);
        gc.restore();
      }
    
    /*
	 * Begin boiler-plate code...
	 * [Animation and events with initialization]
	 */
	public static void main(String[] args) {
		launch(args);
	}
	
	@Override
	public void start(Stage theStage) {
		theStage.setTitle(appName);

		Group root = new Group();
		Scene theScene = new Scene(root);
		theStage.setScene(theScene);

		Canvas canvas = new Canvas(WIDTH, HEIGHT);
		root.getChildren().add(canvas);

		GraphicsContext gc = canvas.getGraphicsContext2D();

		// Initial setup
		initialize();
		
		// Setup and start animation loop (Timeline)
		KeyFrame kf = new KeyFrame(Duration.millis(1000 / FPS),
				e -> {
					// update position
					update();
					// draw frame
					render(gc);
				}
			);
		Timeline mainLoop = new Timeline(kf);
		mainLoop.setCycleCount(Animation.INDEFINITE);
		mainLoop.play();

		theStage.show();
	}
	/*
	 * ... End boiler-plate code
	 */
}